# Introducción al desarrollo ágil con Flask

El objetivo de este tutorial es mostrar el desarrollo del backend para una API REST creada en Python usando el framework Flask y Mongo como base de datos.

Usaremos FlaskPraetorian para securizar la API gracias al uso de JWT y crear filtros para proteger las rutas.

Bibliografía:

* <https://flask-praetorian.readthedocs.io/en/latest/>
* <http://docs.mongoengine.org/projects/flask-mongoengine/en/latest/>
* <https://flask-es.readthedocs.io/patterns/mongoengine/>
* <https://flask.palletsprojects.com/en/2.2.x/quickstart/>
* <https://www.mongodb.com/docs/>
* <https://mongoengine.org/>

## Preparación del entorno

### Mongodb

Instalamos los contenedores de Mongo y Mongo Express, para ello creamos la carpeta mongostack donde vamos a guardar el archivo stack.yml con el contenido siguiente:

```yaml
# Usamos root/example as user/password credentials
version: '3.1'

services:

  mongo:
    image: mongo
    restart: always
    environment:
      MONGO_INITDB_ROOT_USERNAME: root
      MONGO_INITDB_ROOT_PASSWORD: example

  mongo-express:
    image: mongo-express
    restart: always
    ports:
      - 8081:8081
    environment:
      ME_CONFIG_MONGODB_ADMINUSERNAME: root
      ME_CONFIG_MONGODB_ADMINPASSWORD: example
      ME_CONFIG_MONGODB_URL: mongodb://root:example@mongo:27017/
```

Para levantar los contenedores usamos el comando:

```bash
docker-compose -f stack up -d
```

Recuerda que el parámetro **-d** sirve para dejar en *background* el proceso.

### Python y los entornos virtuales

Python permite cerar entornos virtuales que son como pequeñas "máquinas virtuales" donde instalamos y configuramos Python a nuestro antojo y sin ensuciar el sistema. Cuando tenemos varias aplicaciones Python en nuestro equipo, en ocasiones pueden tener paqueters o librerías inconpatibles entre sí (o versiones difrerentes) lo que hace obligatorio su uso.

Para instalar Python en Windows simplemente lo podemos hacer desde el Microsoft Store.

Para instalar Python en Debian/Ubuntu/Mint haremos lo siguiente:

```bash
sudo apt-get install python3.10 python3-pip python3.10-venv
```

Puedes cambiar 3.10 por 3.11 y esta guía funcionará igual si tu sistema lo admite.

Ahora creamos el entorno virtual en la carpeta del proyecto y lo activamos, haciendo en Linux lo siguiente:

```bash
python3 -m venv venv
. ./venv/bin/activate
```

En en Windows:

```PowerShell
.\venv\Scripts\activate
```

Para salvar los paquetes instalados en un archivo para luego reproducir el proceso:

```bash
pip freeze > requitements.txt
```

Para cargar los paquetes (por ejemplo al hacer un git clone del repositorio) tendremos que hacer:

```bash
pip install -r requirements.txt
```

Para desactivar el virtual environment basta con escribir:

```bash
deactivate
```

### Creación del proyecto

Para crear una aplicación Flask debemos tener bien un archivo *wsgi.py* y/o un archivo *app.py* que contiene la lógica principal de la aplicación.

Ejemplo de wsqi.py:

```python
from app import app

if __name__ == "__main__":
  app.run(host='0.0.0.0', port=3000)
```

Ejemplo de app.py:

```python
from flask import Flask

app = Flask(__name__)

@app.route("/")
def hello_world():
    return "<p>Hello, World!</p>"
```
