import flask
from flask import jsonify
from flask import request
import flask_praetorian
from flask_mongoengine import MongoEngine
import flask_cors
import bson

# Initialize flask app for the example
app = flask.Flask(__name__,
            static_url_path='', 
            static_folder='static',
            template_folder='templates')

app.debug = True
app.config["SECRET_KEY"] = "top secret"
app.config["JWT_ACCESS_LIFESPAN"] = {"hours": 7}
app.config["JWT_REFRESH_LIFESPAN"] = {"days": 7}
app.config['MONGODB_SETTINGS'] =  {
            'db': 'inventario',
            'host': 'localhost',
            'username': 'root',
            'password': 'kzljsdr343-sdp98r6-x98',
            'port': 27017
        }

# Inicializa la BBDD
db = MongoEngine()
db.init_app(app)



class User(db.Document):
    _id = db.ObjectIdField()
    username = db.StringField(required=True, unique=True)
    email = db.StringField(required=True)
    first_name = db.StringField(max_length=50)
    last_name = db.StringField(max_length=50)
    hashed_password = db.StringField()
    roles = db.StringField()
    
    is_active = db.BooleanField(required=True)

    def set_password(self, newpwd):
        self.hashed_password = newpwd
    
    @property
    def identity(self):
        """
        *Required Attribute or Property*

        flask-praetorian requires that the user class has an ``identity`` instance
        attribute or property that provides the unique id of the user instance
        """
        return str(self._id)

    @property
    def rolenames(self):
        """
        *Required Attribute or Property*

        flask-praetorian requires that the user class has a ``rolenames`` instance
        attribute or property that provides a list of strings that describe the roles
        attached to the user instance
        """
        try:
            return self.roles.split(",")
        except Exception:
            return []

    @property
    def password(self):
        """
        *Required Attribute or Property*

        flask-praetorian requires that the user class has a ``password`` instance
        attribute or property that provides the hashed password assigned to the user
        instance
        """
        return self.hashed_password

    @classmethod
    def lookup(cls, username):
        """
        *Required Method*

        flask-praetorian requires that the user class implements a ``lookup()``
        class method that takes a single ``username`` argument and returns a user
        instance if there is one that matches or ``None`` if there is not.
        """
        return cls.objects.get(username=username)

    @classmethod
    def get_by_id(cls, in_id):
        if type(in_id) is str:
            in_id = bson.ObjectId(in_id)
        return cls.objects.get(_id=in_id)

    @classmethod
    def identify(cls, id):
        """
        *Required Method*

        flask-praetorian requires that the user class implements an ``identify()``
        class method that takes a single ``id`` argument and returns user instance if
        there is one that matches or ``None`` if there is not.
        """
        if cls.id_exists(id):
            return cls.get_by_id(id)
        return None

    @classmethod
    def id_exists(cls, in_id):
        if type(in_id) is str:
            in_id = bson.ObjectId(in_id)
        return cls.objects(_id=in_id)
    
    def is_valid(self):
        return self.is_active



class Equipos(db.Document):
    _id = db.ObjectIdField()
    nombre = db.StringField()
    procesador = db.StringField()
    ram = db.IntField()
    almacenamiento = db.StringField()
    


# Initialize the flask-praetorian instance for the app
guard = flask_praetorian.Praetorian()
guard.init_app(app, User)

# Initializes CORS so that the api_tool can talk to the example app
cors = flask_cors.CORS()
cors.init_app(app)

# Creamos el usuario por defecto al menos para poder hacer login la primera vez
with app.app_context():
    # db.create_all()
    usuarios = User.objects(username='obijuan')
    if usuarios.count() < 1:
        User(
          username='obijuan',
          email='obijuan@g.educaand.es',
          first_name='Obi',
          last_name='Juan',
          hashed_password=guard.hash_password('starwars'),
          roles='admin',
          is_active = True).save()



# Set up some routes for the example
@app.route("/login-auth", methods=["POST"])
def login():
    """
    Logs a user in by parsing a POST request containing user credentials and
    issuing a JWT token.
    .. example::
       $ curl http://localhost:5000/login-auth -X POST \
         -d '{"username":"obijuan","password":"starwars"}'
    """
    req = flask.request.get_json(force=True)
    username = req.get("username", None)
    password = req.get("password", None)
    user = guard.authenticate(username, password)
    ret = {"access_token": guard.encode_jwt_token(user)}
    return (flask.jsonify(ret), 200)

@app.route('/login-refresh', methods=['POST'])
def loginRefresh():
    """
    Refreshes an existing JWT by creating a new one that is a copy of the old
    except that it has a refreshed access expiration.
    .. example::
       $ curl http://localhost:5000/login-refresh -X POST \
         -H "Authorization: Bearer <your_token>"
    """
    print("refresh request") 
    headers = flask.request.headers
    bearer = headers.get('Authorization')
    old_token = bearer.split()[1]
    new_token = guard.refresh_jwt_token(old_token)
    ret = {'access_token': new_token}
    return ret, 200

@app.route("/api/test")
@flask_praetorian.auth_required
def loginTest():
    """
    A protected test endpoint. It allows to check wether we are logged in or not.
    .. example::
       $ curl http://localhost:5000/api/test -X GET \
         -H "Authorization: Bearer <your_token>"
    """
    return flask.jsonify(
        message="protected endpoint (allowed user {})".format(
            flask_praetorian.current_user().username,
        )
    )

@app.route("/")
def hello_world():
    return "<p>Hello, World!</p>"


@app.route("/api/equipo")
@flask_praetorian.auth_required
def equipoFindAll():
    equipos = Equipos.objects()
    print(equipos)
    return jsonify(equipos), 200


@app.route("/api/equipo", methods=['POST'])
@flask_praetorian.auth_required
def equipoCreate():
    data = request.get_json()
    print(data)
    resultado = Equipos(**data).save()
    print(str(resultado))
    output = {'_id': str(resultado._id)}
    return jsonify(output)


@app.route('/api/equipo/<equipo_id>', methods=['DELETE'])
@flask_praetorian.auth_required
def delete_equipo(equipo_id):
    return jsonify(Equipos.objects(_id=equipo_id).delete())


# Run the APP
if __name__ == "__main__":
    app.run(host="0.0.0.0", port=3000, debug=True)
